(function (parentObject) {
	// Проверяем на повторную инициализацию
    if (parentObject.numTipTable)
        return;

    ////////////////////////////////////////////////
    // Создаем общий интерфейс для создания элементов
    ////////////////////////////////////////////////
    parentObject.numTipTable = {
		////////////////////////////////////
		// function _getTooltip
		////////////////////////////////////
		// Получаем элемент для отображения
		// Функция для внутреннего использования
		////////////////////////////////////
		_getTooltip: function (table) {
			// Если элемент со всплывающем окном для таблицы был уже создан, 
			// то возвращаем его
			if ($(table).data('tipnumid'))
				return $('.table-tipnum.tipnum-' + $(table).data('tipnumid'));

			// Создаем всплывающее окно для таблицы
			var tipId = Math.floor(Math.random() * 1000000),
				numTip = $('<div class="table-tipnum tipnum-' + tipId + '"><p></p></div>');
			// Добавляем в тело страницы
			$('body').append(numTip);
			// Сохраняем идентификатор, чтобы при повторной инициализации
			// использовать уже имеющийся элемент
			$(table).data('tipnumid', tipId);
			return numTip;
		},

		////////////////////////////////////
		// function num
		////////////////////////////////////
		// table - селектор или элемент таблицы
		// options: { - передаваемые параметры
		//  pos - позиция всплывающего окна относительно строки в таблице 
		//		(left, top, right, bottom)
		//  align - при отображении окна сверху или снизу, указывает его месторасположение по горизонтали
		// 		(left, center, right)
		//  valign - при отображении окна справа или слева, указывает его месторасположение по вертикали
		// 		(top, center, bottom)
		//  filter - селектор для выбора нужных строк 
		//		(применяется в случае, если используется скрытые строки в таблице, чтобы исключить их из выборки)
		//  start - номер первого элемента 
		//		(по умолчанию 1)
		// }
		////////////////////////////////////
		num: function (table, options) {
			// Накладываем пользовательские настройки 
			// на настройки по умолчанию
			var _options = $.extend({
				pos: 'left',
				align: 'center', // left, center, right
				valign: 'center', // top, center, bottom
				filter: '*',
				start: 1
			}, options || {}),
				// сохраняем ссылку на элемент управления
				self = this,
				// получаем всплывающее окно
				numTooltip = self._getTooltip(table),
				// собираем строки в таблице
				// (использование tbody обязательно
				//  так как селектор '> tbody > tr' используется для того, 
				//  чтобы не смешивать строки таблицы со внутренними таблицами и заголовками, е
				searchElements = $(table).find('> tbody > tr').filter(_options.filter);
			
			// При наведении мышки, отображаем всплывающее окно
			searchElements.mouseover(function () {
				// Получаем индекс строки в выборке
				var cntItem = searchElements.index(this),
					// Получаем положение элемента относительно окна страницы
					elOffset = $(this).offset(),
					// Сохраняем полностью занимаемое пространство элементом
					elSize = {
						width: $(this).outerWidth(),
						height: $(this).outerHeight()
					};

				// Записываем номер элемента
				numTooltip.find('p').text('' + (_options.start + cntItem));

				// Сохраняем размеры всплывающего окна
				var tipSize = {
						width: numTooltip.outerWidth(),
						height: numTooltip.outerHeight()
					},
					// Определяем набор для сдвига по вертикали
					valign = {
						'top': 0,
						'center': Math.floor((elSize.height - tipSize.height) / 2),
						'bottom': elSize.height - tipSize.height
					},
					// Определяем набор для сдвига по горизонтали
					align = {
						'left': 0,
						'center': Math.floor((elSize.width - tipSize.width) / 2),
						'right': elSize.width - tipSize.width
					};

				// Отображаем всплывающее окно
				numTooltip.css({ opacity: 0.8, display: 'block' });
				
				// Далее указываем место на экране
				if (_options.pos === 'left') {
					numTooltip.css({
						top: elOffset.top + (valign[_options.valign] || 1),
						left: elOffset.left - 5 - tipSize.width
					});
				}
				else if (_options.pos === 'right') {
					numTooltip.css({
						top: elOffset.top + (valign[_options.valign] || 1),
						left: elOffset.left + elSize.width + 5
					});
				}
				else if (_options.pos === 'top') {
					numTooltip.css({
						top: elOffset.top - 5 - tipSize.height,
						left: elOffset.left + (align[_options.align] || 1)
					});
				}
				// Если указана некорректная позиция, то считаем эквивалентным выбору "bottom"
				else /* bottom */{
					numTooltip.css({
						top: elOffset.top + elSize.height + 5,
						left: elOffset.left + (align[_options.align] || 1)
					});
				}
			})
			// При уходе курсора мышки с элемента, скрываем его
			.mouseout(function () {
				numTooltip.css({ display: 'none' });
			});
		}
	}
})(window);